/**
 *
 * fil - Foddex Image Library
 *
 * Copyright (c) 2010 Marc "Foddex" Oude Kotte
 *
 */

#include "fil_internal.h"

//-----------------------------------------------------------------------------
static fil_load_result load_bmp( fil_data_reader& reader, fil_pixels& target, unsigned int ppflags ) {

	// check header
	unsigned short BitmapType;
	reader( BitmapType );
	if (BitmapType!=19778) 
		return FIL_FORMAT_ERROR;

	// get offset of pixel data
	reader.skip( 8 );
	unsigned int PixelOffset;
	reader( PixelOffset );

	// get width and height, and check plane count
	reader.skip( 4 );
	unsigned short planeCount, bitCount;
	int width, height;
	reader( width )( height )( planeCount )( bitCount );
	if (planeCount!=1) 
		return FIL_UNSUPPORTED_SUB_FORMAT;
	if (width<=0 || height<=0)
		return FIL_FORMAT_ERROR;
	if (bitCount!=24 && bitCount!=8 && bitCount!=32)
		return FIL_UNSUPPORTED_SUB_FORMAT;
	unsigned int byteSize = bitCount <=24 ? 3 : 4;

	// get memory
	int size = width * height * byteSize;
	unsigned char* const data = new unsigned char[ size ];

	switch (bitCount) {
		case 32:
		case 24:
			// read pixel data
			reader.seek( PixelOffset );
			reader.read( data, size );
			break;

		case 8:
			// seek to 1024 bytes before pixel offset, that's where palette starts
			unsigned char palette[256][4];
			reader.seek( PixelOffset - sizeof(palette) );
			reader( palette );

			// read 8-bit indices
			unsigned char* const indices = new unsigned char[ width * height ];
			reader.read( indices, width * height );

			// convert to 24-bit data
			unsigned char* cur_pixel = data;
			unsigned char* cur_index = indices;
			for (int y=0; y<height; ++y)
				for (int x=0; x<width; ++x) {

					cur_pixel[0] = palette[ *cur_index ][0];
					cur_pixel[1] = palette[ *cur_index ][1];
					cur_pixel[2] = palette[ *cur_index ][2];

					cur_index += 1;
					cur_pixel += 3;
				}

			// cleanup
			delete [] indices;
			break;
	}

	// set target
	if (reader.in_error_state())
		return FIL_FORMAT_ERROR;
	fil_pixel_format format = byteSize==4 ? FIL_BGRA : FIL_BGR;
	fil_tools::flip_vertical( width, height, data, format );
	fil_tools::install_image( target, reader.get_filename(), width, height, data, format, ppflags );
	return FIL_OK;
}

/** Automatically register "bmp" as a supported format. */
void fil_init_bmp() {

	fil_format_registrar _fil_bmp_registrar( "bmp", load_bmp );
}
