/**
 *
 * fil - Foddex Image Library
 *
 * Copyright (c) 2010 Marc "Foddex" Oude Kotte
 *
 */

#include "fil.h"

/** Data reader. */
class fil_data_reader {

	private:
		const char*		mData;
		const size_t	mSize;
		size_t			mPosition;
		bool			mError;
		std::string		mFilename;

	public:
		fil_data_reader( const char* data, size_t size, const std::string& filename );
		void skip( size_t bytes );
		void seek( size_t bytes );
		bool in_error_state();
	
		void read( void* target, size_t bytes );

		template<typename _T>
		fil_data_reader& operator()( _T& target ) {
			read( &target, sizeof(target) );
			return *this;
		}
		const std::string& get_filename() const;
		const char* get_data() const;
		size_t get_size() const;
};

/** Internal DLL utilities, not available to the outside. */
typedef fil_load_result (*fil_loader)( fil_data_reader&, fil_pixels& target, unsigned int ppflags );
void fil_register_format( const std::string&, fil_loader );
unsigned char fil_get_component_count( fil_pixel_format format );

/** Automatic format registrar */
struct fil_format_registrar {
	fil_format_registrar( const std::string& ext, fil_loader loader ) {
		fil_register_format( ext, loader );
	}
};

/** Tools */
struct fil_tools {

	static void install_image( fil_pixels& target, const std::string& filename, int width, int height, unsigned char* pixel_data, fil_pixel_format format, unsigned int ppflags );
	static void flip_vertical( int width, int height, unsigned char* pixel_data, fil_pixel_format format );
};
