/**
 *
 * fil - Foddex Image Library
 *
 * Copyright (c) 2010 Marc "Foddex" Oude Kotte
 *
 */

#include "fil_internal.h"
#ifdef _MSC_VER
#	include <windows.h>
#endif
#include <GL/gl.h>
#include "GL/glext.h"
#include "fil_opengl.h"

#ifdef _MSC_VER
#pragma warning ( disable : 4996 )	// unsafe STDLIB functions
#endif

GLuint glFilLoadTexture( fil_pixels* fp, GLenum filter, GLenum wrap_s, GLenum wrap_t ) {

	GLenum format;
	switch (fp->get_format()) {
		case FIL_RGB:	format = GL_RGB; break;
		case FIL_BGR:	format = GL_BGR; break;
		case FIL_RGBA:	format = GL_RGBA; break;
		case FIL_BGRA:	format = GL_BGRA; break;
		default:		return 0;
	}

	GLuint tex;
	glGenTextures( 1, &tex );
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, tex );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap_s );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap_t );
	glTexImage2D( GL_TEXTURE_2D, 0, fp->get_bytes_per_pixel(), fp->get_width(), fp->get_height(), 0, format, GL_UNSIGNED_BYTE, fp->get_pixel_data() );
	glDisable( GL_TEXTURE_2D );
	return tex;
}

GLuint glFilLoadTexture( char* image_data, size_t bytes, const std::string& format, unsigned int ppflags, GLenum filter, GLenum wrap_s, GLenum wrap_t ) {

	fil_pixels fp;
	if (fil_load_from_memory( image_data, bytes, format, &fp, ppflags )==FIL_OK)
		return glFilLoadTexture( &fp, filter, wrap_s, wrap_t );
	return 0;
}

GLuint glFilLoadTexture( const std::string& filename, unsigned int ppflags, GLenum filter, GLenum wrap_s, GLenum wrap_t ) {

	fil_pixels fp;
	if (fil_load_from_file( filename, &fp, ppflags )==FIL_OK)
		return glFilLoadTexture( &fp, filter, wrap_s, wrap_t );
	return 0;
}
