/**
 *
 * fil - Foddex Image Library
 *
 * Copyright (c) 2010 Marc "Foddex" Oude Kotte
 *
 */

#include "fil_internal.h"
#include "jpeglib.h"

//-----------------------------------------------------------------------------
static void init_source (j_decompress_ptr cinfo) {

}

//-----------------------------------------------------------------------------
static boolean fill_input_buffer (j_decompress_ptr cinfo) {

	return TRUE;
}

//-----------------------------------------------------------------------------
static void skip_input_data (j_decompress_ptr cinfo, long num_bytes)
{
	struct jpeg_source_mgr *src = cinfo->src;

	if (num_bytes > 0) {
		while (num_bytes > (long) src->bytes_in_buffer) {
			num_bytes -= (long) src->bytes_in_buffer;
			(void) fill_input_buffer(cinfo);
		}
		src->next_input_byte += (size_t) num_bytes;
		src->bytes_in_buffer -= (size_t) num_bytes;
	}
}

//-----------------------------------------------------------------------------
static void term_source (j_decompress_ptr cinfo) {

	cinfo->src->bytes_in_buffer = 0;
	cinfo->src->next_input_byte = 0;
}

//-----------------------------------------------------------------------------
static fil_load_result load_jpeg( fil_data_reader& reader, fil_pixels& target, unsigned int ppflags ) {

	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;
	JSAMPROW row_pointer[1]; 

	// setup decompression structure
	cinfo.err = jpeg_std_error( &jerr );
	jpeg_create_decompress( &cinfo ); 
	
	// setup memory manager, so data is loader from our reader
    cinfo.src = (struct jpeg_source_mgr *) (*cinfo.mem->alloc_small) ((j_common_ptr) &cinfo, JPOOL_PERMANENT, sizeof(struct jpeg_source_mgr));
	memset( cinfo.src, 0, sizeof(*cinfo.src) );
	cinfo.src->bytes_in_buffer = reader.get_size();
	cinfo.src->next_input_byte = reinterpret_cast<const JOCTET*>( reader.get_data() );
	cinfo.src->init_source = init_source;
	cinfo.src->fill_input_buffer = fill_input_buffer;
	cinfo.src->skip_input_data = skip_input_data;
	cinfo.src->term_source = term_source;

	// read header
	jpeg_read_header( &cinfo, TRUE ); 

	// start decompressing
	jpeg_start_decompress( &cinfo );
	fil_pixel_format format;
	switch (cinfo.num_components) {
		case 3:
			format = FIL_RGB;
			break;
		case 4:
			format = FIL_RGBA;
			break;
		default:
			jpeg_finish_decompress( &cinfo );
			jpeg_destroy_decompress( &cinfo );
			return FIL_UNSUPPORTED_SUB_FORMAT;
	}
	unsigned char *raw_image = new unsigned char[ cinfo.output_width*cinfo.output_height*cinfo.num_components ];
	row_pointer[0] = new unsigned char[ cinfo.output_width*cinfo.num_components ];
	unsigned long i;
	unsigned long location = 0; 
	while( cinfo.output_scanline < cinfo.image_height )
	{
		jpeg_read_scanlines( &cinfo, row_pointer, 1 );
		for( i=0; i<cinfo.image_width*cinfo.num_components;i++) 
			raw_image[location++] = row_pointer[0][i];
	}
	jpeg_finish_decompress( &cinfo );
	jpeg_destroy_decompress( &cinfo );
	delete [] row_pointer[0];

	// install image
	fil_tools::install_image( target, reader.get_filename(), cinfo.output_width, cinfo.output_height, raw_image, format, ppflags );
	return FIL_OK;
}

/** Automatically register "jpeg" as a supported format. */
void fil_init_jpeg() {

	fil_format_registrar _fil_jpeg_registrar1( "jpeg", load_jpeg );
	fil_format_registrar _fil_jpeg_registrar2( "jpg", load_jpeg );
}
