/**
 *
 * fil - Foddex Image Library
 *
 * Copyright (c) 2010 Marc "Foddex" Oude Kotte
 *
 */

#include "fil_internal.h"
#include <stdlib.h>
#define ZLIB_WINAPI
#include "png.h"

//-----------------------------------------------------------------------------
void PNGAPI _fil_png_rw_ptr( png_structp png, png_bytep target, png_size_t size ) {

	fil_data_reader& reader = *reinterpret_cast<fil_data_reader*>( png->io_ptr );
	reader.read( target, size );
	if (reader.in_error_state())
		png_error( png, "failed to read" );
}

//-----------------------------------------------------------------------------
static fil_load_result load_png( fil_data_reader& reader, fil_pixels& target, unsigned int ppflags ) {

	// check signature
	char sig[8];
	reader( sig );
	if (!png_check_sig((unsigned char *) sig, 8))
		return FIL_FORMAT_ERROR;

	// create read struct
	png_structp png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL );
	if (!png_ptr)
		return FIL_GENERIC_ERROR;
	png_infop info_ptr = png_create_info_struct( png_ptr );
	if (!info_ptr) {
		png_destroy_read_struct( &png_ptr, (png_infopp) NULL, (png_infopp) NULL );
		return FIL_GENERIC_ERROR;
	}

	// handle libpng errors, then check whether the PNG file had a bKGD chunk
	if (setjmp(png_jmpbuf(png_ptr))) {
		png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
		return FIL_GENERIC_ERROR;
	}

	// setup our own IO functions
	png_set_read_fn( png_ptr, &reader, _fil_png_rw_ptr );
	png_set_sig_bytes( png_ptr, 8 );

	// read header, setup transformations so that we have 24 or 32 bits per pixel
	png_uint_32 width, height;
	int bit_depth, color_type;
	png_read_info( png_ptr, info_ptr );
	png_get_IHDR( png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, NULL, NULL, NULL );
	/*if (color_type & PNG_COLOR_MASK_ALPHA)
		png_set_strip_alpha( png_ptr );
		// enable this to strip alpha info, always getting a RGB texture
		*/
	if (bit_depth > 8)
		png_set_strip_16( png_ptr );
	if (color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
		png_set_gray_to_rgb( png_ptr );
	if (color_type == PNG_COLOR_TYPE_PALETTE)
		png_set_palette_to_rgb( png_ptr );
	png_read_update_info(png_ptr, info_ptr);
	png_get_IHDR( png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, NULL, NULL, NULL );
	if (color_type != PNG_COLOR_TYPE_RGB && color_type != PNG_COLOR_TYPE_RGBA) {
		png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
		return FIL_GENERIC_ERROR;
	}
	fil_pixel_format format = color_type == PNG_COLOR_TYPE_RGB ? FIL_RGB : FIL_RGBA; 

	// prepare for reading pixels
	unsigned int rowbytes = png_get_rowbytes(png_ptr, info_ptr);
	unsigned char* image_data;
	if (!(image_data = new unsigned char[ rowbytes * height])) {
		png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
		return FIL_MEMORY_ALLOCATION_ERROR;
	}
	png_bytepp row_pointers;
	if ((row_pointers = (png_bytepp)malloc(height*sizeof(png_bytep))) == NULL) {
        png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
        free( image_data );
        return FIL_MEMORY_ALLOCATION_ERROR;
    }
    for (unsigned int i = 0;  i < height;  ++i)
        row_pointers[i] = image_data + i*rowbytes;

	// read image
    png_read_image(png_ptr, row_pointers);

	// cleanup
    free( row_pointers );
	png_destroy_read_struct( &png_ptr, &info_ptr, NULL );

	// setup
	fil_tools::install_image( target, reader.get_filename(), width, height, image_data, format, ppflags );
	return FIL_OK;
}

/** Automatically register "png" as a supported format. */
void fil_init_png() {

	fil_format_registrar _fil_png_registrar( "png", load_png );
}
