/**
 *
 * fil - Foddex Image Library
 *
 * Copyright (c) 2010 Marc "Foddex" Oude Kotte
 *
 */

#include "fil_internal.h"

//-----------------------------------------------------------------------------
static fil_load_result load_tga( fil_data_reader& reader, fil_pixels& target, unsigned int ppflags ) {

	#define TGA_TYPE_COLOR 2
	#define TGA_TYPE_COLOR_RLE 10
	#define TGA_DESC_VERTICAL 0x20

	// read targa type
	unsigned char type;
	reader.skip( 2 );
	reader( type );
	reader.skip( 9 );

	// read size from bitmap
	short w, h;
	reader( w )( h );
	if (w<=0 || h<=0) 
		return FIL_FORMAT_ERROR;
	int width, height;
	width = w;
	height = h;

	//  read bit count and image descriptor from file
	unsigned char bitCount, imageDesc;
	reader( bitCount )( imageDesc );

	// check for unsupported formats
	if (type != TGA_TYPE_COLOR && type != TGA_TYPE_COLOR_RLE) 
		return FIL_UNSUPPORTED_SUB_FORMAT;
	if (bitCount!=24 && bitCount!=32) 
		return FIL_UNSUPPORTED_SUB_FORMAT;

	// allocate memory for material data
	fil_pixel_format format;
	if (bitCount==24)
		format = FIL_BGR;
	else if (bitCount==32)
		format = FIL_BGRA;
	else
		return FIL_UNSUPPORTED_SUB_FORMAT;
	int compCount = fil_get_component_count( format );

	// get memory
	unsigned int size = (unsigned int)(width) * (unsigned int)(height) * compCount;
	unsigned char* data = new unsigned char[ size ];
	unsigned char* pixels = data;

	// parse bitmap data
	switch (type) {
		case TGA_TYPE_COLOR: {
			// read pixels straight from stream
			reader.read( data, size );
			break;
		}

		case TGA_TYPE_COLOR_RLE:
			// parse RLE compression
			int x=0;
			unsigned char rgba[4];
			while (x < w*h) {
				// read RLE value
				unsigned char l;
				reader( l );

				// if RLE bit is set
				if(l & 0x80) {
					l &= 0x7F;
					reader.read( rgba, compCount );
					for (int i=0; i<l+1; ++i) {
						memcpy( pixels, rgba, compCount );
						pixels += compCount;
					}
				} else {
					l &= 0x7F;
					for (int i=0; i<l+1; i++) {
						reader.read( rgba, compCount );
						memcpy( pixels, rgba, compCount );
						pixels += compCount;
					}
				}
				x += l+1;
			}
			break;
	}
	if (reader.in_error_state())
		return FIL_FORMAT_ERROR;

	// check if we need to y-flip the image
	if ((imageDesc & TGA_DESC_VERTICAL)==0)
		fil_tools::flip_vertical( width, height, data, format );
	fil_tools::install_image( target, reader.get_filename(), width, height, data, format, ppflags );
	return FIL_OK;
}

/** Automatically register "tga" as a supported format. */
void fil_init_tga() {

	fil_format_registrar _fil_tga_registrar( "tga", load_tga );
}

